unit uFenster;
(* ******************************************************************** *)
(* K L A S S E : FENSTER                                                *)
(* -------------------------------------------------------------------- *)
(* Version     : 1.4 - Vollversion                                      *)
(* Autor       : (c) S. Spolwig - OSZ Handel I, Berlin 10997            *)
(* Beschreibung: Fenster fuer das Suchprogramm Such99                   *)
(*                                                                      *)
(* Compiler    : Delphi 3.0                                             *)
(* Aenderungen : V. 1.2   03-JAN-99   Lineares Suchen                   *)
(*                  1.3   03-JAN-99   Binaeres Suchen                   *)
(*                  1.31              Vollversion                       *)
(*                  1.32  04-MAR-00   jedes El. einmal                  *)
(*                  1.4   16-FEB-02   Bezeichner                        *)  
(* ******************************************************************** *)

interface
(* ==================================================================== *)
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls,
  uWortGenerator,
  uWortListe, Buttons;

type
  TFensterFrm = class(TForm)
    AusgabeLbox      : TListBox;
    SchluesselEdt    : TEdit;
    SchluesselLbl    : TLabel;
    EinstellungenGBox: TGroupBox;
    ListenlaengeRGrp : TRadioGroup;
    HaeufigkeitCBox  : TCheckBox;
    UPCaseCBox       : TCheckBox;
    WortLaengeEdt    : TEdit;
    WortLaengeLbl    : TLabel;
    AuswahlRGrp      : TRadioGroup;
    StatisktikPnl    : TPanel;
    TrefferposLbl    : TLabel;
    Label1           : TLabel;
    Label2           : TLabel;
    VergleicheLbl    : TLabel;
    Fuellstand       : TProgressBar;
    SteuerPnl: TPanel;
    OKBtn: TButton;
    EndeBtn: TButton;
    SuchBtn: TButton;


    procedure FormCreate(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure SchluesselEdtKeyPress(Sender: TObject; var Key: Char);
    procedure OKBtnClick(Sender: TObject);
    procedure SuchBtnClick(Sender: TObject);

   private
    procedure Init;
    procedure SuchListeFuellen;
    procedure MaskeAktualisieren;
    procedure DatenAktualisieren;
   public
 end;

var
  FensterFrm : TFensterFrm;
  Suchliste  : TWortListe;
  Kunstwort  : TWort;

  AnzahlElemente : integer;


IMPLEMENTATION
(* ==================================================================== *)
{$R *.DFM}

procedure TFensterFrm.Init;
(* -------------------------------------------------------------------- *)
begin
  AusgabeLBox.ItemIndex      := -1;  // nichts angezeigt
  ListenlaengeRGrp.ItemIndex := 0;
  HaeufigkeitCBox.Checked    := false;
  UPCaseCBox.Checked         := false;
end;


procedure TFensterFrm.FormCreate(Sender: TObject);
(* -------------------------------------------------------------------- *)
begin
  Kunstwort := TWort.Create;
  Kunstwort.Init;
  Suchliste := TWortListe.Create;
  Suchliste.Init;
  Init;
end;


procedure TFensterFrm.SchluesselEdtKeyPress(Sender: TObject; var Key: Char);
(* -------------------------------------------------------------------- *)
(* Aufgabe: In der Liste nach einem Schluessel suchen und in der Ausgabe*)
(*          Listbox anzeigen.                                           *)
(*          Suchpos aus der Suchliste muss um 1 vermindert werden, weil *)
(*          AusgabeLBox mit 0 anfaengt                                  *)
(* -------------------------------------------------------------------- *)
var ergebnis : integer;

begin
  AusgabeLBox.ItemIndex := -1;  // alten Eintrag loeschen
  if key = #13
  then
    begin
     case AuswahlRGrp.ItemIndex of
      0 : begin  // Linear
           AusgabeLBox.ItemIndex := Suchliste.SearchPos(SchluesselEdt.Text) - 1;
           TrefferPosLbl.Caption := inttostr(Suchliste.SearchPos(SchluesselEdt.Text));
          end;

      1 : begin  // Binaer
            AusgabeLBox.ItemIndex := Suchliste.SearchPosBIN(SchluesselEdt.Text) - 1;
          // Suchliste.RekursivDurchsuchen(SchluesselEdt.Text,1,AnzahlElemente,ergebnis) ;
          // AusgabeLBox.ItemIndex := ergebnis - 1;
           TrefferPosLbl.Caption := inttostr(Suchliste.SearchPosBIN(SchluesselEdt.Text));
          end;
      end;

      VergleicheLbl.Caption := inttostr(Suchliste.GetVergleiche);
    end;
end;


procedure TFensterFrm.EndeBtnClick(Sender: TObject);
(* -------------------------------------------------------------------- *)
begin
  AusgabeLBox.Clear;
  Suchliste.Free;
  Close;
end;


procedure TFensterFrm.OKBtnClick(Sender: TObject);
(* -------------------------------------------------------------------- *)
begin
  AusgabeLBox.Clear;  // alten Schrott wegputzen
  if ListenLaengeRGrp.ItemIndex = 0   // Lilaenge 25
  then
     begin
      DatenAktualisieren;
      MaskeAktualisieren;
    end
  else       // mehr als 25 und jedes nur einmal
    begin
      if (HaeufigkeitCBox.Checked = true) and (ListenLaengeRGrp.ItemIndex > 0) and
      ((strtoint(WortLaengeEdt.Text) < 3))
      then
        begin
         if Application.MessageBox('Wortlnge muss grer als 2 sein'+ #13 + 'Bitte neu eingeben',
           'Eingabefehler', MB_ICONWARNING + MB_OK) = IDOK then ;
         end
      else
        begin
          DatenAktualisieren;
          MaskeAktualisieren;
        end;
    end;
end;


procedure TFensterFrm.SuchListeFuellen;
(* -------------------------------------------------------------------- *)
var
  i       : integer;
  element : string;

begin
  Fuellstand.Max := AnzahlElemente;
  Suchliste.Init; //  erst alte Liste leeren

  for i := 1 to AnzahlElemente do
  begin
    if (HaeufigkeitCBox.Checked = true) // jedes El. nur einmal
    then
      repeat
        Kunstwort.Erzeugen;
        element := Kunstwort.GetWort;
      until  Suchliste.SearchFor(element) = 0
     else
      begin
        Kunstwort.Erzeugen;
        element := Kunstwort.GetWort;
      end;
    Suchliste.Append(element);

    Fuellstand.Stepit;
  end;

//  Fuellstand.Position := 0;
  if AuswahlRGrp.ItemIndex = 1
  then  Suchliste.Sort;
end;



procedure TFensterFrm.MaskeAktualisieren;
(* -------------------------------------------------------------------- *)
var i : integer;
begin
  AusgabeLBox.Clear;
  Suchliste.First;

  for i := 1 to Suchliste.GetLen do
  begin
    AusgabeLBox.items.add(Suchliste.getelement);
    Suchliste.Next;
  end;
  
  AusgabeLBox.ItemIndex := -1;  // nichts angezeigt
  VergleicheLbl.Caption := '';
  TrefferposLbl.Caption := '';

  Fuellstand.Position := 0;
end;


procedure TFensterFrm.DatenAktualisieren;
(* -------------------------------------------------------------------- *)
begin
  Kunstwort.SetLaenge(strtoint(WortLaengeEdt.Text));
  Kunstwort.SetUpCase(UpCaseCBox.checked);

  case ListenLaengeRGrp.ItemIndex of
      0 : AnzahlElemente :=   25;
      1 : AnzahlElemente :=  100;
      2 : AnzahlElemente := 1000;
      3 : AnzahlElemente :=10000;
  end;

  SuchlisteFuellen;
end;

procedure TFensterFrm.SuchBtnClick(Sender: TObject);
(* -------------------------------------------------------------------- *)
var ergebnis : integer;

begin
  AusgabeLBox.ItemIndex := -1;  // alten Eintrag loeschen
     case AuswahlRGrp.ItemIndex of
      0 : begin  // Linear
           AusgabeLBox.ItemIndex := Suchliste.SearchPos(SchluesselEdt.Text) - 1;
           TrefferPosLbl.Caption := inttostr(Suchliste.SearchPos(SchluesselEdt.Text));
          end;

      1 : begin  // Binaer
            AusgabeLBox.ItemIndex := Suchliste.SearchPosBIN(SchluesselEdt.Text) - 1;
          // Suchliste.RekursivDurchsuchen(SchluesselEdt.Text,1,AnzahlElemente,ergebnis) ;
          // AusgabeLBox.ItemIndex := ergebnis - 1;
           TrefferPosLbl.Caption := inttostr(Suchliste.SearchPosBIN(SchluesselEdt.Text));
          end;
      end;

      VergleicheLbl.Caption := inttostr(Suchliste.GetVergleiche);
end;

END.
