UNIT uWortListe;
(* ******************************************************************** *)
(*                                                                      *)
(* K L A S S E   : TWortListe                                           *)
(* -------------------------------------------------------------------- *)
(* Version       : 1.0                                                  *)
(* Autor         : S. Spolwig, OSZ-Handel I, 10997 Berlin               *)
(*                                                                      *)
(* Aufgabe       : Statische Liste zur Verwaltung beliebiger strings    *)
(*                                                                      *)
(* Compiler      : Delphi 6.0                                           *)
(* Aenderung     : V. 1.0   - 16-FEB-02   erbt TStrListe                *)
(* ******************************************************************** *)

INTERFACE
(* ==================================================================== *)
uses uStrListe;

type
  TWortListe = class (TStrListe)
   public
    function  SearchPos (key: string) : integer;            virtual;
    function  SearchPosBin (key: string) : integer;         virtual;
    procedure RekursivDurchsuchen (key : string; unten,oben : integer;
                                    var Pos: integer ); virtual;
    function  GetVergleiche :  integer;                     virtual;
  end;
(*  ----------------- B e s c h r e i b u n g -------------------------

Oberklasse     : TStrListe
Bezugsklassen  : -

Methoden
--------

SearchPos (key: string) : integer
  Anfrage: ob key in der Liste enthalten ist
  vorher : -
  nachher: liefert die Position in der Liste; wenn nicht vorhanden, 0


SearchPosBin (key: string) : integer
  Anfrage: ob key in der Liste enthalten ist
  vorher : -
  nachher: liefert die Position in der Liste; wenn nicht vorhanden, 0


RekursivDurchsuchen (key : string; unten,oben : integer;
                    var Pos: integer )
----------------------------------------------------------------------- *)


IMPLEMENTATION
(* ==================================================================== *)

var
  Vergleiche : integer;  // beim Suchen

{
function  TWortListe.SearchPos (key : string) : integer;
(* -------------------------------------------------------------------- *)
(* Auftrag: lineares Suchen wie blich                                  *)
(* -------------------------------------------------------------------- *)
begin
  Result     := 0;   // nicht gefunden!
  Vergleiche := 0;

  First;
  while (Result = 0) AND (NOT EoList) do
  begin
     if Kollektion[AktuellePos] = key
     then Result := AktuellePos
     else inc(AktuellePos);   //Next;
     inc(Vergleiche);
  end;
end;

 }

function  TWortListe.SearchPos (key : string) : integer;
(* -------------------------------------------------------------------- *)
(* Auftrag: lineares Suchen nach Wirth                                  *)
(* -------------------------------------------------------------------- *)
begin
  Vergleiche := 1;
  Result     := 0;  // nicht drin

  AktuellePos := Listenlaenge + 1;
  SetElement(key);
  First;
  while Kollektion[AktuellePos] <> key do
  begin
    inc(AktuellePos); // next;
    inc(Vergleiche);
  end;

  if AktuellePos <= ListenLaenge
  then Result := AktuellePos;
end;


function  TWortListe.SearchPosBin (key : string) : integer;
(* -------------------------------------------------------------------- *)
(* Auftrag: Binaeres Suchen                                             *)
(* -------------------------------------------------------------------- *)
var
  unten,
  oben,
  mitte   : integer;

begin
  Result     := 0;   // nicht gefunden
  Vergleiche := 0;
  unten      := 1;
  oben       := ListenLaenge;

  while (oben >= unten) and (Result = 0) do
  begin
     mitte := (unten + oben) div 2;
     inc(Vergleiche);
     if (Kollektion[mitte] = key)      (* Treffer *)
     then Result := mitte
     else
       begin
         if (key < Kollektion[mitte])
         then oben  := mitte - 1
         else unten := mitte + 1;
//         else oben  := mitte - 1;
       end;
  end; (* while *)
end;


procedure TWortListe.RekursivDurchsuchen (key : string; unten,oben : integer;
                                      var Pos: integer );
(* -------------------------------------------------------------------- *)
(* Bemerkung: Vergleiche sind nicht mitgezaehlt, damit die Parameter-   *)
(*            leiste nicht noch lnger wird                             *)
(* -------------------------------------------------------------------- *)
var mitte : integer;

begin
   if unten > oben
   then Pos := 0    (* abbruch - nicht gefunden *)
   else
     begin
       mitte := (unten + oben) div 2;

       if Kollektion[mitte] = key
       then Pos := mitte  (* Treffer ! *)
       else
         if (key < Kollektion[mitte])
         then RekursivDurchsuchen(key, unten, mitte -1, Pos)
         else RekursivDurchsuchen(key, mitte + 1, oben, Pos)
//         else RekursivDurchsuchen(key, unten, mitte -1, Pos);
     end;  (* else AktuellePos *)
end;


function  TWortListe.GetVergleiche :  integer;
(* -------------------------------------------------------------------- *)
begin
   Result := Vergleiche;
end;

END.
